"""Generated implementation of table_monitoring."""

# WARNING DO NOT EDIT
# This code was generated from table-monitoring.mcn

from __future__ import annotations

import abc  # noqa: F401
import dataclasses  # noqa: F401
import datetime  # noqa: F401
import enum  # noqa: F401
import isodate  # noqa: F401
import json  # noqa: F401
import jsonschema  # noqa: F401
import logging  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401

from ..cluster import ClusterId
from ..feature_store_run import ExecutionStatistics
from ..jobs import RunStatus, TableMonitoringJobId, TableMonitoringRunId
from ..schedule import Schedule, ScheduleState
from ..summary_statistics import SummaryStatistics
from ..table import TableId, TableVersionId


@dataclasses.dataclass(frozen=True)
class TableMonitoringJobVersionId:
    """Unique identifier for a specific version of a table monitoring job.
    
    Args:
        value (uuid.UUID): A data field.
    """
    
    value: uuid.UUID
    
    def __str__(self) -> str:
        """Return a str of the wrapped value."""
        return str(self.value)
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for TableMonitoringJobVersionId data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "string",
            "format": "uuid"
        }
    
    @classmethod
    def from_json(cls, data: dict) -> TableMonitoringJobVersionId:
        """Validate and parse JSON data into an instance of TableMonitoringJobVersionId.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of TableMonitoringJobVersionId.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return TableMonitoringJobVersionId(uuid.UUID(hex=data))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing TableMonitoringJobVersionId", exc_info=ex)
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return str(self.value)
    
    @classmethod
    def from_json_key(cls, data: str) -> TableMonitoringJobVersionId:
        """Parse a JSON string such as a dictionary key."""
        return TableMonitoringJobVersionId((lambda s: uuid.UUID(hex=s))(data))
    
    def to_json_key(self) -> str:
        """Serialise as a JSON string suitable for use as a dictionary key."""
        return str(self.value)


@dataclasses.dataclass(frozen=True)
class TableMonitoringJobName:
    """Unique name of a table monitoring job.
    
    Args:
        value (str): A data field.
    """
    
    value: str
    
    def __str__(self) -> str:
        """Return a str of the wrapped value."""
        return str(self.value)
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for TableMonitoringJobName data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "string"
        }
    
    @classmethod
    def from_json(cls, data: dict) -> TableMonitoringJobName:
        """Validate and parse JSON data into an instance of TableMonitoringJobName.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of TableMonitoringJobName.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return TableMonitoringJobName(str(data))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing TableMonitoringJobName", exc_info=ex)
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return str(self.value)
    
    @classmethod
    def from_json_key(cls, data: str) -> TableMonitoringJobName:
        """Parse a JSON string such as a dictionary key."""
        return TableMonitoringJobName(str(data))
    
    def to_json_key(self) -> str:
        """Serialise as a JSON string suitable for use as a dictionary key."""
        return str(self.value)


@dataclasses.dataclass(frozen=True)
class MonitoringResultId:
    """Unique identifier of a table monitoring result.
    
    Args:
        value (int): A data field.
    """
    
    value: int
    
    def __str__(self) -> str:
        """Return a str of the wrapped value."""
        return str(self.value)
    
    def __int__(self) -> int:
        """Return an int of the wrapped value."""
        return int(self.value)
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for MonitoringResultId data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "integer"
        }
    
    @classmethod
    def from_json(cls, data: dict) -> MonitoringResultId:
        """Validate and parse JSON data into an instance of MonitoringResultId.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of MonitoringResultId.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return MonitoringResultId(int(data))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing MonitoringResultId", exc_info=ex)
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return self.value
    
    @classmethod
    def from_json_key(cls, data: str) -> MonitoringResultId:
        """Parse a JSON string such as a dictionary key."""
        return MonitoringResultId(int(data))
    
    def to_json_key(self) -> str:
        """Serialise as a JSON string suitable for use as a dictionary key."""
        return str(self.value)


@dataclasses.dataclass(frozen=True)
class TableMonitoringJobCreationRequest:
    """Request to create a new table monitoring job.
    
    Args:
        name (TableMonitoringJobName): A data field.
        description (str): A data field.
        tables (typing.List[TableId]): A data field.
        schedule (Schedule): A data field.
        cluster (ClusterId): A data field.
        enabled (bool): A data field.
    """
    
    name: TableMonitoringJobName
    description: str
    tables: typing.List[TableId]
    schedule: Schedule
    cluster: ClusterId
    enabled: bool
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for TableMonitoringJobCreationRequest data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "name": TableMonitoringJobName.json_schema(),
                "description": {
                    "type": "string"
                },
                "tables": {
                    "type": "array",
                    "item": TableId.json_schema()
                },
                "schedule": Schedule.json_schema(),
                "cluster": ClusterId.json_schema(),
                "enabled": {
                    "type": "boolean"
                }
            },
            "required": [
                "name",
                "description",
                "tables",
                "schedule",
                "cluster",
                "enabled",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> TableMonitoringJobCreationRequest:
        """Validate and parse JSON data into an instance of TableMonitoringJobCreationRequest.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of TableMonitoringJobCreationRequest.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return TableMonitoringJobCreationRequest(
                name=TableMonitoringJobName.from_json(data["name"]),
                description=str(data["description"]),
                tables=[TableId.from_json(v) for v in data["tables"]],
                schedule=Schedule.from_json(data["schedule"]),
                cluster=ClusterId.from_json(data["cluster"]),
                enabled=bool(data["enabled"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing TableMonitoringJobCreationRequest",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "name": self.name.to_json(),
            "description": str(self.description),
            "tables": [v.to_json() for v in self.tables],
            "schedule": self.schedule.to_json(),
            "cluster": self.cluster.to_json(),
            "enabled": self.enabled
        }


@dataclasses.dataclass(frozen=True)
class TableMonitoringJob:
    """Details of a table monitoring job.
    
    Args:
        id (TableMonitoringJobId): A data field.
        version (TableMonitoringJobVersionId): A data field.
        name (TableMonitoringJobName): A data field.
        description (str): A data field.
        tables (typing.List[TableId]): A data field.
        created (datetime.datetime): A data field.
        schedule (Schedule): A data field.
        cluster (ClusterId): A data field.
        enabled (bool): A data field.
    """
    
    id: TableMonitoringJobId
    version: TableMonitoringJobVersionId
    name: TableMonitoringJobName
    description: str
    tables: typing.List[TableId]
    created: datetime.datetime
    schedule: Schedule
    cluster: ClusterId
    enabled: bool
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for TableMonitoringJob data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "id": TableMonitoringJobId.json_schema(),
                "version": TableMonitoringJobVersionId.json_schema(),
                "name": TableMonitoringJobName.json_schema(),
                "description": {
                    "type": "string"
                },
                "tables": {
                    "type": "array",
                    "item": TableId.json_schema()
                },
                "created": {
                    "type": "string",
                    "format": "date-time"
                },
                "schedule": Schedule.json_schema(),
                "cluster": ClusterId.json_schema(),
                "enabled": {
                    "type": "boolean"
                }
            },
            "required": [
                "id",
                "version",
                "name",
                "description",
                "tables",
                "created",
                "schedule",
                "cluster",
                "enabled",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> TableMonitoringJob:
        """Validate and parse JSON data into an instance of TableMonitoringJob.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of TableMonitoringJob.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return TableMonitoringJob(
                id=TableMonitoringJobId.from_json(data["id"]),
                version=TableMonitoringJobVersionId.from_json(data["version"]),
                name=TableMonitoringJobName.from_json(data["name"]),
                description=str(data["description"]),
                tables=[TableId.from_json(v) for v in data["tables"]],
                created=isodate.parse_datetime(data["created"]),
                schedule=Schedule.from_json(data["schedule"]),
                cluster=ClusterId.from_json(data["cluster"]),
                enabled=bool(data["enabled"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing TableMonitoringJob",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "id": self.id.to_json(),
            "version": self.version.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "tables": [v.to_json() for v in self.tables],
            "created": self.created.strftime('%Y-%m-%dT%H:%M:%S.%f%z'),
            "schedule": self.schedule.to_json(),
            "cluster": self.cluster.to_json(),
            "enabled": self.enabled
        }


@dataclasses.dataclass(frozen=True)
class TableMonitoringRunCreationRequest:
    """Request to create a new table monitoring job run.
    
    Args:
        jobId (TableMonitoringJobId): A data field.
        jobVersionId (TableMonitoringJobVersionId): A data field.
        status (RunStatus): A data field.
        error (typing.Optional[str]): A data field.
        scheduleState (typing.Optional[ScheduleState]): A data field.
        executionStatistics (typing.Optional[ExecutionStatistics]): A data field.
    """
    
    jobId: TableMonitoringJobId
    jobVersionId: TableMonitoringJobVersionId
    status: RunStatus
    error: typing.Optional[str]
    scheduleState: typing.Optional[ScheduleState]
    executionStatistics: typing.Optional[ExecutionStatistics]
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for TableMonitoringRunCreationRequest data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "jobId": TableMonitoringJobId.json_schema(),
                "jobVersionId": TableMonitoringJobVersionId.json_schema(),
                "status": RunStatus.json_schema(),
                "error": {
                    "oneOf": [
                        {"type": "null"},
                        {"type": "string"},
                    ]
                },
                "scheduleState": {
                    "oneOf": [
                        {"type": "null"},
                        ScheduleState.json_schema(),
                    ]
                },
                "executionStatistics": {
                    "oneOf": [
                        {"type": "null"},
                        ExecutionStatistics.json_schema(),
                    ]
                }
            },
            "required": [
                "jobId",
                "jobVersionId",
                "status",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> TableMonitoringRunCreationRequest:
        """Validate and parse JSON data into an instance of TableMonitoringRunCreationRequest.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of TableMonitoringRunCreationRequest.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return TableMonitoringRunCreationRequest(
                jobId=TableMonitoringJobId.from_json(data["jobId"]),
                jobVersionId=TableMonitoringJobVersionId.from_json(data["jobVersionId"]),
                status=RunStatus.from_json(data["status"]),
                error=(lambda v: v and str(v))(data.get("error", None)),
                scheduleState=(
                    lambda v: v and ScheduleState.from_json(v)
                )(
                    data.get("scheduleState", None)
                ),
                executionStatistics=(
                    lambda v: v and ExecutionStatistics.from_json(v)
                )(
                    data.get("executionStatistics", None)
                ),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing TableMonitoringRunCreationRequest",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "jobId": self.jobId.to_json(),
            "jobVersionId": self.jobVersionId.to_json(),
            "status": self.status.to_json(),
            "error": (lambda v: v and str(v))(self.error),
            "scheduleState": (lambda v: v and v.to_json())(self.scheduleState),
            "executionStatistics": (lambda v: v and v.to_json())(self.executionStatistics)
        }


@dataclasses.dataclass(frozen=True)
class TableMonitoringJobRun:
    """Details of a table monitoring job run.
    
    Args:
        id (TableMonitoringRunId): A data field.
        jobId (TableMonitoringJobId): A data field.
        jobVersionId (TableMonitoringJobVersionId): A data field.
        created (datetime.datetime): A data field.
        status (RunStatus): A data field.
        error (typing.Optional[str]): A data field.
        scheduleState (typing.Optional[ScheduleState]): A data field.
        executionStatistics (typing.Optional[ExecutionStatistics]): A data field.
    """
    
    id: TableMonitoringRunId
    jobId: TableMonitoringJobId
    jobVersionId: TableMonitoringJobVersionId
    created: datetime.datetime
    status: RunStatus
    error: typing.Optional[str]
    scheduleState: typing.Optional[ScheduleState]
    executionStatistics: typing.Optional[ExecutionStatistics]
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for TableMonitoringJobRun data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "id": TableMonitoringRunId.json_schema(),
                "jobId": TableMonitoringJobId.json_schema(),
                "jobVersionId": TableMonitoringJobVersionId.json_schema(),
                "created": {
                    "type": "string",
                    "format": "date-time"
                },
                "status": RunStatus.json_schema(),
                "error": {
                    "oneOf": [
                        {"type": "null"},
                        {"type": "string"},
                    ]
                },
                "scheduleState": {
                    "oneOf": [
                        {"type": "null"},
                        ScheduleState.json_schema(),
                    ]
                },
                "executionStatistics": {
                    "oneOf": [
                        {"type": "null"},
                        ExecutionStatistics.json_schema(),
                    ]
                }
            },
            "required": [
                "id",
                "jobId",
                "jobVersionId",
                "created",
                "status",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> TableMonitoringJobRun:
        """Validate and parse JSON data into an instance of TableMonitoringJobRun.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of TableMonitoringJobRun.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return TableMonitoringJobRun(
                id=TableMonitoringRunId.from_json(data["id"]),
                jobId=TableMonitoringJobId.from_json(data["jobId"]),
                jobVersionId=TableMonitoringJobVersionId.from_json(data["jobVersionId"]),
                created=isodate.parse_datetime(data["created"]),
                status=RunStatus.from_json(data["status"]),
                error=(lambda v: v and str(v))(data.get("error", None)),
                scheduleState=(
                    lambda v: v and ScheduleState.from_json(v)
                )(
                    data.get("scheduleState", None)
                ),
                executionStatistics=(
                    lambda v: v and ExecutionStatistics.from_json(v)
                )(
                    data.get("executionStatistics", None)
                ),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing TableMonitoringJobRun",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "id": self.id.to_json(),
            "jobId": self.jobId.to_json(),
            "jobVersionId": self.jobVersionId.to_json(),
            "created": self.created.strftime('%Y-%m-%dT%H:%M:%S.%f%z'),
            "status": self.status.to_json(),
            "error": (lambda v: v and str(v))(self.error),
            "scheduleState": (lambda v: v and v.to_json())(self.scheduleState),
            "executionStatistics": (lambda v: v and v.to_json())(self.executionStatistics)
        }


@dataclasses.dataclass(frozen=True)
class MonitoringResultCreationRequest:
    """Request to record monitoring results from a monitoring run.
    
    Args:
        table (TableId): A data field.
        tableVersion (TableVersionId): A data field.
        job (TableMonitoringJobId): A data field.
        run (TableMonitoringRunId): A data field.
        partition (str): A data field.
        count (int): A data field.
        columnStatistics (typing.List[SummaryStatistics]): A data field.
    """
    
    table: TableId
    tableVersion: TableVersionId
    job: TableMonitoringJobId
    run: TableMonitoringRunId
    partition: str
    count: int
    columnStatistics: typing.List[SummaryStatistics]
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for MonitoringResultCreationRequest data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "table": TableId.json_schema(),
                "tableVersion": TableVersionId.json_schema(),
                "job": TableMonitoringJobId.json_schema(),
                "run": TableMonitoringRunId.json_schema(),
                "partition": {
                    "type": "string"
                },
                "count": {
                    "type": "integer"
                },
                "columnStatistics": {
                    "type": "array",
                    "item": SummaryStatistics.json_schema()
                }
            },
            "required": [
                "table",
                "tableVersion",
                "job",
                "run",
                "partition",
                "count",
                "columnStatistics",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> MonitoringResultCreationRequest:
        """Validate and parse JSON data into an instance of MonitoringResultCreationRequest.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of MonitoringResultCreationRequest.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return MonitoringResultCreationRequest(
                table=TableId.from_json(data["table"]),
                tableVersion=TableVersionId.from_json(data["tableVersion"]),
                job=TableMonitoringJobId.from_json(data["job"]),
                run=TableMonitoringRunId.from_json(data["run"]),
                partition=str(data["partition"]),
                count=int(data["count"]),
                columnStatistics=[SummaryStatistics.from_json(v) for v in data["columnStatistics"]],
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing MonitoringResultCreationRequest",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "table": self.table.to_json(),
            "tableVersion": self.tableVersion.to_json(),
            "job": self.job.to_json(),
            "run": self.run.to_json(),
            "partition": str(self.partition),
            "count": self.count,
            "columnStatistics": [v.to_json() for v in self.columnStatistics]
        }


@dataclasses.dataclass(frozen=True)
class MonitoringResultPartial:
    """Partial details of a monitoring result.
    
    This class omits the large fields containing detailed monitoring data to
    reduce the amount of data transferred from the server.
    
    Args:
        id (MonitoringResultId): A data field.
        table (TableId): A data field.
        tableVersion (TableVersionId): A data field.
        job (TableMonitoringJobId): A data field.
        run (TableMonitoringRunId): A data field.
        partition (str): A data field.
        count (int): A data field.
    """
    
    id: MonitoringResultId
    table: TableId
    tableVersion: TableVersionId
    job: TableMonitoringJobId
    run: TableMonitoringRunId
    partition: str
    count: int
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for MonitoringResultPartial data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "id": MonitoringResultId.json_schema(),
                "table": TableId.json_schema(),
                "tableVersion": TableVersionId.json_schema(),
                "job": TableMonitoringJobId.json_schema(),
                "run": TableMonitoringRunId.json_schema(),
                "partition": {
                    "type": "string"
                },
                "count": {
                    "type": "integer"
                }
            },
            "required": [
                "id",
                "table",
                "tableVersion",
                "job",
                "run",
                "partition",
                "count",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> MonitoringResultPartial:
        """Validate and parse JSON data into an instance of MonitoringResultPartial.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of MonitoringResultPartial.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return MonitoringResultPartial(
                id=MonitoringResultId.from_json(data["id"]),
                table=TableId.from_json(data["table"]),
                tableVersion=TableVersionId.from_json(data["tableVersion"]),
                job=TableMonitoringJobId.from_json(data["job"]),
                run=TableMonitoringRunId.from_json(data["run"]),
                partition=str(data["partition"]),
                count=int(data["count"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing MonitoringResultPartial",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "id": self.id.to_json(),
            "table": self.table.to_json(),
            "tableVersion": self.tableVersion.to_json(),
            "job": self.job.to_json(),
            "run": self.run.to_json(),
            "partition": str(self.partition),
            "count": self.count
        }


@dataclasses.dataclass(frozen=True)
class MonitoringResult:
    """Detailed monitoring results from a table monitoring run.
    
    Instances of this class contain the full results from a table monitoring
    job run. As such, they can be very large. Unless you need to use the
    statistical results computed during the monitoring run, you should use
    MonitoringResultPartial instead.
    
    Args:
        id (MonitoringResultId): A data field.
        table (TableId): A data field.
        tableVersion (TableVersionId): A data field.
        job (TableMonitoringJobId): A data field.
        run (TableMonitoringRunId): A data field.
        partition (str): A data field.
        count (int): A data field.
        columnStatistics (typing.List[SummaryStatistics]): A data field.
    """
    
    id: MonitoringResultId
    table: TableId
    tableVersion: TableVersionId
    job: TableMonitoringJobId
    run: TableMonitoringRunId
    partition: str
    count: int
    columnStatistics: typing.List[SummaryStatistics]
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for MonitoringResult data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "id": MonitoringResultId.json_schema(),
                "table": TableId.json_schema(),
                "tableVersion": TableVersionId.json_schema(),
                "job": TableMonitoringJobId.json_schema(),
                "run": TableMonitoringRunId.json_schema(),
                "partition": {
                    "type": "string"
                },
                "count": {
                    "type": "integer"
                },
                "columnStatistics": {
                    "type": "array",
                    "item": SummaryStatistics.json_schema()
                }
            },
            "required": [
                "id",
                "table",
                "tableVersion",
                "job",
                "run",
                "partition",
                "count",
                "columnStatistics",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> MonitoringResult:
        """Validate and parse JSON data into an instance of MonitoringResult.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of MonitoringResult.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return MonitoringResult(
                id=MonitoringResultId.from_json(data["id"]),
                table=TableId.from_json(data["table"]),
                tableVersion=TableVersionId.from_json(data["tableVersion"]),
                job=TableMonitoringJobId.from_json(data["job"]),
                run=TableMonitoringRunId.from_json(data["run"]),
                partition=str(data["partition"]),
                count=int(data["count"]),
                columnStatistics=[SummaryStatistics.from_json(v) for v in data["columnStatistics"]],
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing MonitoringResult",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "id": self.id.to_json(),
            "table": self.table.to_json(),
            "tableVersion": self.tableVersion.to_json(),
            "job": self.job.to_json(),
            "run": self.run.to_json(),
            "partition": str(self.partition),
            "count": self.count,
            "columnStatistics": [v.to_json() for v in self.columnStatistics]
        }



"""Generated implementation of event."""

# WARNING DO NOT EDIT
# This code was generated from event.mcn

from __future__ import annotations

import abc  # noqa: F401
import dataclasses  # noqa: F401
import datetime  # noqa: F401
import enum  # noqa: F401
import isodate  # noqa: F401
import json  # noqa: F401
import jsonschema  # noqa: F401
import logging  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401

from ..commit import CommitId
from ..feature_store import FeatureStoreId
from ..jobs import (
    FeatureStoreRunId, RunStatus, TableMonitoringJobId, TableMonitoringRunId
)
from ..merge_request import MergeRequestCommentId, MergeRequestId
from ..table_caching import TableCachingJobId, TableCachingRunId


@dataclasses.dataclass(frozen=True)
class Event(abc.ABC):
    """Event notifications sent to registered webhooks."""
    
    ADT_TYPE: typing.ClassVar[str] = ""
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    @classmethod
    def json_schema(cls) -> Event:
        """JSON schema for variant Event.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        adt_types = [klass.ADT_TYPE for klass in cls.__subclasses__()]
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": adt_types
                }
            },
            "required": [
                "adt_type",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> Event:
        """Validate and parse JSON data into an instance of Event.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of Event.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            adt_type = data.get("adt_type", None)
            for klass in cls.__subclasses__():
                if klass.ADT_TYPE == adt_type:
                    return klass.from_json(data)
            raise ValueError("Unknown adt_type: '{ty}'".format(ty=adt_type))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing Event", exc_info=ex)
            raise
    
    @abc.abstractmethod
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        raise NotImplementedError


@dataclasses.dataclass(frozen=True)
class NewMergeRequestEvent(Event):
    """A new merge request has been created.
    
    Args:
        id (MergeRequestId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "newmergerequest"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: MergeRequestId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for NewMergeRequestEvent data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": MergeRequestId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> NewMergeRequestEvent:
        """Validate and parse JSON data into an instance of NewMergeRequestEvent.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of NewMergeRequestEvent.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return NewMergeRequestEvent(
                id=MergeRequestId.from_json(data["id"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing NewMergeRequestEvent",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json()
        }


@dataclasses.dataclass(frozen=True)
class NewMergeRequestCommentEvent(Event):
    """A comment has been posted on a merge request.
    
    Args:
        id (MergeRequestId): A data field.
        commentId (MergeRequestCommentId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "newmergerequestcomment"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: MergeRequestId
    commentId: MergeRequestCommentId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for NewMergeRequestCommentEvent data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": MergeRequestId.json_schema(),
                "commentId": MergeRequestCommentId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "commentId",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> NewMergeRequestCommentEvent:
        """Validate and parse JSON data into an instance of NewMergeRequestCommentEvent.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of NewMergeRequestCommentEvent.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return NewMergeRequestCommentEvent(
                id=MergeRequestId.from_json(data["id"]),
                commentId=MergeRequestCommentId.from_json(data["commentId"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing NewMergeRequestCommentEvent",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "commentId": self.commentId.to_json()
        }


@dataclasses.dataclass(frozen=True)
class MergeRequestAcceptedEvent(Event):
    """A merge request has been accepted.
    
    Args:
        id (MergeRequestId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "mergerequestaccepted"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: MergeRequestId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for MergeRequestAcceptedEvent data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": MergeRequestId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> MergeRequestAcceptedEvent:
        """Validate and parse JSON data into an instance of MergeRequestAcceptedEvent.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of MergeRequestAcceptedEvent.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return MergeRequestAcceptedEvent(
                id=MergeRequestId.from_json(data["id"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing MergeRequestAcceptedEvent",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json()
        }


@dataclasses.dataclass(frozen=True)
class NewCommitEvent(Event):
    """A new commit has been created.
    
    Args:
        id (CommitId): A data field.
        branch (typing.Optional[str]): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "newcommit"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: CommitId
    branch: typing.Optional[str]
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for NewCommitEvent data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": CommitId.json_schema(),
                "branch": {
                    "oneOf": [
                        {"type": "null"},
                        {"type": "string"},
                    ]
                }
            },
            "required": [
                "adt_type",
                "id",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> NewCommitEvent:
        """Validate and parse JSON data into an instance of NewCommitEvent.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of NewCommitEvent.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return NewCommitEvent(
                id=CommitId.from_json(data["id"]),
                branch=(lambda v: v and str(v))(data.get("branch", None)),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing NewCommitEvent",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "branch": (lambda v: v and str(v))(self.branch)
        }


@dataclasses.dataclass(frozen=True)
class FeatureStoreRunEvent(Event):
    """A feature store run has changed status.
    
    Args:
        featureStore (FeatureStoreId): A data field.
        run (FeatureStoreRunId): A data field.
        status (RunStatus): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "featurestorerun"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    featureStore: FeatureStoreId
    run: FeatureStoreRunId
    status: RunStatus
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for FeatureStoreRunEvent data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "featureStore": FeatureStoreId.json_schema(),
                "run": FeatureStoreRunId.json_schema(),
                "status": RunStatus.json_schema()
            },
            "required": [
                "adt_type",
                "featureStore",
                "run",
                "status",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> FeatureStoreRunEvent:
        """Validate and parse JSON data into an instance of FeatureStoreRunEvent.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of FeatureStoreRunEvent.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return FeatureStoreRunEvent(
                featureStore=FeatureStoreId.from_json(data["featureStore"]),
                run=FeatureStoreRunId.from_json(data["run"]),
                status=RunStatus.from_json(data["status"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing FeatureStoreRunEvent",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "featureStore": self.featureStore.to_json(),
            "run": self.run.to_json(),
            "status": self.status.to_json()
        }


@dataclasses.dataclass(frozen=True)
class MonitoringRunEvent(Event):
    """A monitoring run has changed status.
    
    Args:
        job (TableMonitoringJobId): A data field.
        run (TableMonitoringRunId): A data field.
        status (RunStatus): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "monitoringrun"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    job: TableMonitoringJobId
    run: TableMonitoringRunId
    status: RunStatus
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for MonitoringRunEvent data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "job": TableMonitoringJobId.json_schema(),
                "run": TableMonitoringRunId.json_schema(),
                "status": RunStatus.json_schema()
            },
            "required": [
                "adt_type",
                "job",
                "run",
                "status",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> MonitoringRunEvent:
        """Validate and parse JSON data into an instance of MonitoringRunEvent.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of MonitoringRunEvent.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return MonitoringRunEvent(
                job=TableMonitoringJobId.from_json(data["job"]),
                run=TableMonitoringRunId.from_json(data["run"]),
                status=RunStatus.from_json(data["status"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing MonitoringRunEvent",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "job": self.job.to_json(),
            "run": self.run.to_json(),
            "status": self.status.to_json()
        }


@dataclasses.dataclass(frozen=True)
class CachingRunEvent(Event):
    """A table caching run has changed status.
    
    Args:
        job (TableCachingJobId): A data field.
        run (TableCachingRunId): A data field.
        status (RunStatus): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "cachingrun"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    job: TableCachingJobId
    run: TableCachingRunId
    status: RunStatus
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for CachingRunEvent data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "job": TableCachingJobId.json_schema(),
                "run": TableCachingRunId.json_schema(),
                "status": RunStatus.json_schema()
            },
            "required": [
                "adt_type",
                "job",
                "run",
                "status",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> CachingRunEvent:
        """Validate and parse JSON data into an instance of CachingRunEvent.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of CachingRunEvent.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return CachingRunEvent(
                job=TableCachingJobId.from_json(data["job"]),
                run=TableCachingRunId.from_json(data["run"]),
                status=RunStatus.from_json(data["status"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing CachingRunEvent",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "job": self.job.to_json(),
            "run": self.run.to_json(),
            "status": self.status.to_json()
        }



"""Generated implementation of commit."""

# WARNING DO NOT EDIT
# This code was generated from commit.mcn

from __future__ import annotations

import abc  # noqa: F401
import dataclasses  # noqa: F401
import datetime  # noqa: F401
import enum  # noqa: F401
import isodate  # noqa: F401
import json  # noqa: F401
import jsonschema  # noqa: F401
import logging  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401

from ..user import UserId


@dataclasses.dataclass(frozen=True)
class CommitId:
    """Unique identifier of a commit.
    
    Args:
        value (uuid.UUID): A data field.
    """
    
    value: uuid.UUID
    
    def __str__(self) -> str:
        """Return a str of the wrapped value."""
        return str(self.value)
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for CommitId data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "string",
            "format": "uuid"
        }
    
    @classmethod
    def from_json(cls, data: dict) -> CommitId:
        """Validate and parse JSON data into an instance of CommitId.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of CommitId.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return CommitId(uuid.UUID(hex=data))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing CommitId", exc_info=ex)
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return str(self.value)
    
    @classmethod
    def from_json_key(cls, data: str) -> CommitId:
        """Parse a JSON string such as a dictionary key."""
        return CommitId((lambda s: uuid.UUID(hex=s))(data))
    
    def to_json_key(self) -> str:
        """Serialise as a JSON string suitable for use as a dictionary key."""
        return str(self.value)


@dataclasses.dataclass(frozen=True)
class Commit:
    """Metadata of a commit.
    
    Args:
        id (CommitId): A data field.
        parents (typing.List[CommitId]): A data field.
        createdAt (datetime.datetime): A data field.
        author (UserId): A data field.
        description (typing.Optional[str]): A data field.
    """
    
    id: CommitId
    parents: typing.List[CommitId]
    createdAt: datetime.datetime
    author: UserId
    description: typing.Optional[str]
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for Commit data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "id": CommitId.json_schema(),
                "parents": {
                    "type": "array",
                    "item": CommitId.json_schema()
                },
                "createdAt": {
                    "type": "string",
                    "format": "date-time"
                },
                "author": UserId.json_schema(),
                "description": {
                    "oneOf": [
                        {"type": "null"},
                        {"type": "string"},
                    ]
                }
            },
            "required": [
                "id",
                "parents",
                "createdAt",
                "author",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> Commit:
        """Validate and parse JSON data into an instance of Commit.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of Commit.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return Commit(
                id=CommitId.from_json(data["id"]),
                parents=[CommitId.from_json(v) for v in data["parents"]],
                createdAt=isodate.parse_datetime(data["createdAt"]),
                author=UserId.from_json(data["author"]),
                description=(lambda v: v and str(v))(data.get("description", None)),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing Commit",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "id": self.id.to_json(),
            "parents": [v.to_json() for v in self.parents],
            "createdAt": self.createdAt.strftime('%Y-%m-%dT%H:%M:%S.%f%z'),
            "author": self.author.to_json(),
            "description": (lambda v: v and str(v))(self.description)
        }



"""Generated implementation of source."""

# WARNING DO NOT EDIT
# This code was generated from source.mcn

from __future__ import annotations

import abc  # noqa: F401
import dataclasses  # noqa: F401
import datetime  # noqa: F401
import enum  # noqa: F401
import isodate  # noqa: F401
import json  # noqa: F401
import jsonschema  # noqa: F401
import logging  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401

from ..attribute import Attribute, SensitiveAttribute
from ..credentials_provider_config import CredentialsProviderConfig
from ..file_format import FileFormat
from ..label import Label


@dataclasses.dataclass(frozen=True)
class SourceId:
    """Unique identifier for a source data store.
    
    Args:
        value (int): A data field.
    """
    
    value: int
    
    def __str__(self) -> str:
        """Return a str of the wrapped value."""
        return str(self.value)
    
    def __int__(self) -> int:
        """Return an int of the wrapped value."""
        return int(self.value)
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for SourceId data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "integer"
        }
    
    @classmethod
    def from_json(cls, data: dict) -> SourceId:
        """Validate and parse JSON data into an instance of SourceId.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of SourceId.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return SourceId(int(data))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing SourceId", exc_info=ex)
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return int(self.value)
    
    @classmethod
    def from_json_key(cls, data: str) -> SourceId:
        """Parse a JSON string such as a dictionary key."""
        return SourceId(int(data))
    
    def to_json_key(self) -> str:
        """Serialise as a JSON string suitable for use as a dictionary key."""
        return str(self.value)


@dataclasses.dataclass(frozen=True)
class SourceVersionId:
    """Unique identifier of a particular version of a source data store.
    
    Args:
        value (uuid.UUID): A data field.
    """
    
    value: uuid.UUID
    
    def __str__(self) -> str:
        """Return a str of the wrapped value."""
        return str(self.value)
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for SourceVersionId data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "string",
            "format": "uuid"
        }
    
    @classmethod
    def from_json(cls, data: dict) -> SourceVersionId:
        """Validate and parse JSON data into an instance of SourceVersionId.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of SourceVersionId.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return SourceVersionId(uuid.UUID(hex=data))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing SourceVersionId", exc_info=ex)
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return str(self.value)
    
    @classmethod
    def from_json_key(cls, data: str) -> SourceVersionId:
        """Parse a JSON string such as a dictionary key."""
        return SourceVersionId((lambda s: uuid.UUID(hex=s))(data))
    
    def to_json_key(self) -> str:
        """Serialise as a JSON string suitable for use as a dictionary key."""
        return str(self.value)


@dataclasses.dataclass(frozen=True)
class SourceName:
    """Unique name of a source data store.
    
    Args:
        value (str): A data field.
    """
    
    value: str
    
    def __str__(self) -> str:
        """Return a str of the wrapped value."""
        return str(self.value)
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for SourceName data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "string"
        }
    
    @classmethod
    def from_json(cls, data: dict) -> SourceName:
        """Validate and parse JSON data into an instance of SourceName.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of SourceName.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return SourceName(str(data))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing SourceName", exc_info=ex)
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return str(self.value)
    
    @classmethod
    def from_json_key(cls, data: str) -> SourceName:
        """Parse a JSON string such as a dictionary key."""
        return SourceName(str(data))
    
    def to_json_key(self) -> str:
        """Serialise as a JSON string suitable for use as a dictionary key."""
        return str(self.value)


@dataclasses.dataclass(frozen=True)
class Source(abc.ABC):
    """Details for source data stores.
    
    Args:
        attributes (typing.List[Attribute]): A data field.
        description (str): A data field.
        id (SourceId): A data field.
        labels (typing.List[Label]): A data field.
        name (SourceName): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = ""
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    attributes: typing.List[Attribute]
    description: str
    id: SourceId
    labels: typing.List[Label]
    name: SourceName
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> Source:
        """JSON schema for variant Source.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        adt_types = [klass.ADT_TYPE for klass in cls.__subclasses__()]
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": adt_types
                }
            },
            "required": [
                "adt_type",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> Source:
        """Validate and parse JSON data into an instance of Source.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of Source.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            adt_type = data.get("adt_type", None)
            for klass in cls.__subclasses__():
                if klass.ADT_TYPE == adt_type:
                    return klass.from_json(data)
            raise ValueError("Unknown adt_type: '{ty}'".format(ty=adt_type))
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug("Invalid JSON data received while parsing Source", exc_info=ex)
            raise
    
    @abc.abstractmethod
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        raise NotImplementedError


@dataclasses.dataclass(frozen=True)
class S3Source(Source):
    """An input data source on AWS S3.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        bucket (str): A data field.
        path (str): A data field.
        fileFormat (FileFormat): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "s3"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    bucket: str
    path: str
    fileFormat: FileFormat
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for S3Source data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "bucket": {
                    "type": "string"
                },
                "path": {
                    "type": "string"
                },
                "fileFormat": FileFormat.json_schema(),
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "bucket",
                "path",
                "fileFormat",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> S3Source:
        """Validate and parse JSON data into an instance of S3Source.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of S3Source.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return S3Source(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                bucket=str(data["bucket"]),
                path=str(data["path"]),
                fileFormat=FileFormat.from_json(data["fileFormat"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing S3Source",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "bucket": str(self.bucket),
            "path": str(self.path),
            "fileFormat": self.fileFormat.to_json(),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class S3ASource(Source):
    """An input data source on S3-compatible object storage.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        bucket (str): A data field.
        path (str): A data field.
        fileFormat (FileFormat): A data field.
        endpoint (str): A data field.
        accessKey (str): A data field.
        secretKey (str): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "s3a"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    bucket: str
    path: str
    fileFormat: FileFormat
    endpoint: str
    accessKey: str
    secretKey: str
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for S3ASource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "bucket": {
                    "type": "string"
                },
                "path": {
                    "type": "string"
                },
                "fileFormat": FileFormat.json_schema(),
                "endpoint": {
                    "type": "string"
                },
                "accessKey": {
                    "type": "string"
                },
                "secretKey": {
                    "type": "string"
                },
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "bucket",
                "path",
                "fileFormat",
                "endpoint",
                "accessKey",
                "secretKey",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> S3ASource:
        """Validate and parse JSON data into an instance of S3ASource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of S3ASource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return S3ASource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                bucket=str(data["bucket"]),
                path=str(data["path"]),
                fileFormat=FileFormat.from_json(data["fileFormat"]),
                endpoint=str(data["endpoint"]),
                accessKey=str(data["accessKey"]),
                secretKey=str(data["secretKey"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing S3ASource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "bucket": str(self.bucket),
            "path": str(self.path),
            "fileFormat": self.fileFormat.to_json(),
            "endpoint": str(self.endpoint),
            "accessKey": str(self.accessKey),
            "secretKey": str(self.secretKey),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class JDBCSource(Source):
    """An input data source using JDBC.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        url (str): A data field.
        schema (str): A data field.
        credentialsProvider (CredentialsProviderConfig): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "jdbc"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    url: str
    schema: str
    credentialsProvider: CredentialsProviderConfig
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for JDBCSource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "url": {
                    "type": "string"
                },
                "schema": {
                    "type": "string"
                },
                "credentialsProvider": CredentialsProviderConfig.json_schema(),
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "url",
                "schema",
                "credentialsProvider",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> JDBCSource:
        """Validate and parse JSON data into an instance of JDBCSource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of JDBCSource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return JDBCSource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                url=str(data["url"]),
                schema=str(data["schema"]),
                credentialsProvider=CredentialsProviderConfig.from_json(data["credentialsProvider"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing JDBCSource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "url": str(self.url),
            "schema": str(self.schema),
            "credentialsProvider": self.credentialsProvider.to_json(),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class HiveSource(Source):
    """An input data source on Hive.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        database (str): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "hive"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    database: str
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for HiveSource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "database": {
                    "type": "string"
                },
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "database",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> HiveSource:
        """Validate and parse JSON data into an instance of HiveSource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of HiveSource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return HiveSource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                database=str(data["database"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing HiveSource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "database": str(self.database),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class BigQuerySource(Source):
    """An input data source on Google BigQuery.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        path (str): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "bigquery"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    path: str
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for BigQuerySource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "path": {
                    "type": "string"
                },
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "path",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> BigQuerySource:
        """Validate and parse JSON data into an instance of BigQuerySource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of BigQuerySource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return BigQuerySource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                path=str(data["path"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing BigQuerySource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "path": str(self.path),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class GCSSource(Source):
    """An input data source on Google Cloud Storage.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        bucket (str): A data field.
        path (str): A data field.
        fileFormat (FileFormat): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "gcs"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    bucket: str
    path: str
    fileFormat: FileFormat
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for GCSSource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "bucket": {
                    "type": "string"
                },
                "path": {
                    "type": "string"
                },
                "fileFormat": FileFormat.json_schema(),
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "bucket",
                "path",
                "fileFormat",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> GCSSource:
        """Validate and parse JSON data into an instance of GCSSource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of GCSSource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return GCSSource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                bucket=str(data["bucket"]),
                path=str(data["path"]),
                fileFormat=FileFormat.from_json(data["fileFormat"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing GCSSource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "bucket": str(self.bucket),
            "path": str(self.path),
            "fileFormat": self.fileFormat.to_json(),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class LocalSource(Source):
    """An input data source on the local filesystem.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        path (str): A data field.
        fileFormat (FileFormat): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "local"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    path: str
    fileFormat: FileFormat
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for LocalSource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "path": {
                    "type": "string"
                },
                "fileFormat": FileFormat.json_schema(),
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "path",
                "fileFormat",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> LocalSource:
        """Validate and parse JSON data into an instance of LocalSource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of LocalSource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return LocalSource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                path=str(data["path"]),
                fileFormat=FileFormat.from_json(data["fileFormat"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing LocalSource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "path": str(self.path),
            "fileFormat": self.fileFormat.to_json(),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class HDFSSource(Source):
    """An input data source on HDFS.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        path (str): A data field.
        fileFormat (FileFormat): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "hdfs"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    path: str
    fileFormat: FileFormat
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for HDFSSource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "path": {
                    "type": "string"
                },
                "fileFormat": FileFormat.json_schema(),
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "path",
                "fileFormat",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> HDFSSource:
        """Validate and parse JSON data into an instance of HDFSSource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of HDFSSource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return HDFSSource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                path=str(data["path"]),
                fileFormat=FileFormat.from_json(data["fileFormat"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing HDFSSource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "path": str(self.path),
            "fileFormat": self.fileFormat.to_json(),
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class KafkaSource(Source):
    """An input data source on Kafka.
    
    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        bootstrapServers (str): A data field.
        schemaRegistryUrl (str): A data field.
        kafkaPropertiesProviders (typing.List[SensitiveAttribute]): A data field.
        version (SourceVersionId): A data field.
    """
    
    ADT_TYPE: typing.ClassVar[str] = "kafka"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)
    
    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    bootstrapServers: str
    schemaRegistryUrl: str
    kafkaPropertiesProviders: typing.List[SensitiveAttribute]
    version: SourceVersionId
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for KafkaSource data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "bootstrapServers": {
                    "type": "string"
                },
                "schemaRegistryUrl": {
                    "type": "string"
                },
                "kafkaPropertiesProviders": {
                    "type": "array",
                    "item": SensitiveAttribute.json_schema()
                },
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "bootstrapServers",
                "schemaRegistryUrl",
                "kafkaPropertiesProviders",
                "version",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> KafkaSource:
        """Validate and parse JSON data into an instance of KafkaSource.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of KafkaSource.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return KafkaSource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                bootstrapServers=str(data["bootstrapServers"]),
                schemaRegistryUrl=str(data["schemaRegistryUrl"]),
                kafkaPropertiesProviders=[SensitiveAttribute.from_json(v) for v in data["kafkaPropertiesProviders"]],
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing KafkaSource",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "bootstrapServers": str(self.bootstrapServers),
            "schemaRegistryUrl": str(self.schemaRegistryUrl),
            "kafkaPropertiesProviders": [v.to_json() for v in self.kafkaPropertiesProviders],
            "version": self.version.to_json()
        }


@dataclasses.dataclass(frozen=True)
class SnowflakeSource(Source):
    """An input data source on Snowflake.

    Args:
        id (SourceId): A data field.
        name (SourceName): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        url (str): A data field.
        warehouse (str): A data field.
        database (str): A data field.
        schema (str): A data field.
        credentialsProvider (CredentialsProviderConfig): A data field.
        version (SourceVersionId): A data field.
    """

    ADT_TYPE: typing.ClassVar[str] = "snowflake"
    adt_type: str = dataclasses.field(init=False, repr=False, default=ADT_TYPE)

    id: SourceId
    name: SourceName
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    url: str
    warehouse: str
    database: str
    schema: str
    credentialsProvider: CredentialsProviderConfig
    version: SourceVersionId

    @classmethod
    def json_schema(cls):
        """Return the JSON schema for SnowflakeSource data.

        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "adt_type": {
                    "type": "string",
                    "enum": [cls.ADT_TYPE]
                },
                "id": SourceId.json_schema(),
                "name": SourceName.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "url": {
                    "type": "string"
                },
                "warehouse": {
                    "type": "string"
                },
                "database": {
                    "type": "string"
                },
                "schema": {
                    "type": "string"
                },
                "credentialsProvider": CredentialsProviderConfig.json_schema(),
                "version": SourceVersionId.json_schema()
            },
            "required": [
                "adt_type",
                "id",
                "name",
                "description",
                "labels",
                "attributes",
                "url",
                "warehouse",
                "database",
                "schema",
                "credentialsProvider",
                "version",
            ]
        }

    @classmethod
    def from_json(cls, data: dict) -> SnowflakeSource:
        """Validate and parse JSON data into an instance of SnowflakeSource.

        Args:
            data (dict): JSON data to validate and parse.

        Returns:
            An instance of SnowflakeSource.

        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return SnowflakeSource(
                id=SourceId.from_json(data["id"]),
                name=SourceName.from_json(data["name"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                url=str(data["url"]),
                warehouse=str(data["warehouse"]),
                database=str(data["database"]),
                schema=str(data["schema"]),
                credentialsProvider=CredentialsProviderConfig.from_json(data["credentialsProvider"]),
                version=SourceVersionId.from_json(data["version"]),
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing SnowflakeSource",
                exc_info=ex
            )
            raise

    def to_json(self) -> dict:
        """Serialise this instance as JSON.

        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "adt_type": self.ADT_TYPE,
            "id": self.id.to_json(),
            "name": self.name.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "url": str(self.url),
            "warehouse": str(self.warehouse),
            "database": str(self.database),
            "schema": str(self.schema),
            "credentialsProvider": self.credentialsProvider.to_json(),
            "version": self.version.to_json()
        }



"""Generated implementation of feature_set_creation_request."""

# WARNING DO NOT EDIT
# This code was generated from feature-set-creation-request.mcn

from __future__ import annotations

import abc  # noqa: F401
import dataclasses  # noqa: F401
import datetime  # noqa: F401
import enum  # noqa: F401
import isodate  # noqa: F401
import json  # noqa: F401
import jsonschema  # noqa: F401
import logging  # noqa: F401
import typing  # noqa: F401
import uuid  # noqa: F401

from ..attribute import Attribute
from ..entity import EntityId
from ..feature_id import FeatureId
from ..feature_set import FeatureSetName
from ..label import Label


@dataclasses.dataclass(frozen=True)
class FeatureSetCreationRequest:
    """Request to create a new feature set.
    
    Args:
        name (FeatureSetName): A data field.
        entity (EntityId): A data field.
        description (str): A data field.
        labels (typing.List[Label]): A data field.
        attributes (typing.List[Attribute]): A data field.
        features (typing.List[FeatureId]): A data field.
    """
    
    name: FeatureSetName
    entity: EntityId
    description: str
    labels: typing.List[Label]
    attributes: typing.List[Attribute]
    features: typing.List[FeatureId]
    
    @classmethod
    def json_schema(cls) -> dict:
        """Return the JSON schema for FeatureSetCreationRequest data.
        
        Returns:
            A Python dictionary describing the JSON schema.
        """
        return {
            "type": "object",
            "properties": {
                "name": FeatureSetName.json_schema(),
                "entity": EntityId.json_schema(),
                "description": {
                    "type": "string"
                },
                "labels": {
                    "type": "array",
                    "item": Label.json_schema()
                },
                "attributes": {
                    "type": "array",
                    "item": Attribute.json_schema()
                },
                "features": {
                    "type": "array",
                    "item": FeatureId.json_schema()
                }
            },
            "required": [
                "name",
                "entity",
                "description",
                "labels",
                "attributes",
                "features",
            ]
        }
    
    @classmethod
    def from_json(cls, data: dict) -> FeatureSetCreationRequest:
        """Validate and parse JSON data into an instance of FeatureSetCreationRequest.
        
        Args:
            data (dict): JSON data to validate and parse.
        
        Returns:
            An instance of FeatureSetCreationRequest.
        
        Raises:
            ValidationError: When schema validation fails.
            KeyError: When a required field is missing from the JSON.
        """
        try:
            jsonschema.validate(data, cls.json_schema())
            return FeatureSetCreationRequest(
                name=FeatureSetName.from_json(data["name"]),
                entity=EntityId.from_json(data["entity"]),
                description=str(data["description"]),
                labels=[Label.from_json(v) for v in data["labels"]],
                attributes=[Attribute.from_json(v) for v in data["attributes"]],
                features=[FeatureId.from_json(v) for v in data["features"]],
            )
        except jsonschema.exceptions.ValidationError as ex:
            logging.debug(
                "Invalid JSON data received while parsing FeatureSetCreationRequest",
                exc_info=ex
            )
            raise
    
    def to_json(self) -> dict:
        """Serialise this instance as JSON.
        
        Returns:
            Data ready to serialise as JSON.
        """
        return {
            "name": self.name.to_json(),
            "entity": self.entity.to_json(),
            "description": str(self.description),
            "labels": [v.to_json() for v in self.labels],
            "attributes": [v.to_json() for v in self.attributes],
            "features": [v.to_json() for v in self.features]
        }
